from __future__ import print_function
from builtins import map
from builtins import object
import json
import pkg_resources
import os.path
import bisect
# import pdb

home_path = os.path.expanduser("~")
config_file = home_path+"/.journalabbrev/db.json"
config_file_exists = os.path.exists(config_file)

if config_file_exists:
    db_path = config_file
else:
    db_path = pkg_resources.resource_filename("journalabbrev",
                                              "data/db_abbrev.json")


class Db_abbrev(object):
    def __init__(self):
        self.db_file = db_path
        self.updates_db = []
        self.open()

    def open(self):
        with open(self.db_file) as file_data:
            self.db = json.load(file_data)
            self.db_names = [
                item["name"].lower().replace(" ", "")
                for item in self.db
            ]
            self.db_abbrevs = [
                item["abbrev"].lower().replace(" ", "")
                for item in self.db
            ]

    def insert(self, name, abbrev):
        return self.updates_db.append({
            "name": name,
            "abbrev": abbrev,
            "what": "insert"
        })

    def update(self, name, abbrev):
        return self.updates_db.append({
            "name": name,
            "abbrev": abbrev,
            "what": "update"
        })

    def get_index(self, value, key="name"):
        value = value.lower().replace(" ", "")

        if key == "name":
            index = bisect.bisect(self.db_names, value) - 1
            if self.db_names[index] == value:
                return index
            else:
                return -1

        try:
            index = self.db_abbrevs.index(value)
        except ValueError:
            index = -1

        return index

    def find_closet_index(self, name):
        name = name.lower().replace(" ", "")

        index = bisect.bisect(self.db_names, name)

        return index

    def close(self):
        def action(data):
            what = data["what"]
            data.pop("what", None)

            if what == "insert":
                index = self.find_closet_index(data["name"])
    #avoid duplication
                if self.db_names[index-1] != data["name"]:
                    self.db.insert(index, data)
            else:
                index = self.get_index(data["name"])

                if index != -1:
                    self.db[index]["abbrev"] = data["abbrev"]

        if len(self.updates_db) > 0:

            list(map(action, self.updates_db))

            if not config_file_exists:
                os.makedirs(os.path.dirname(config_file))
                self.db_file = config_file

            with open(self.db_file, "w") as file_data:
                json.dump(self.db, file_data)

            print("Databse was updated")



from __future__ import print_function
from builtins import input
from itertools import groupby
import operator
from functools import reduce


def manual_update_in(bibs, db_abbrev):
    actions = {
        "y": lambda: update_in(bibs, db_abbrev),
        "n": lambda: bibs,
        "c": lambda: update_in(bibs, db_abbrev, custom=True)
    }
    question = "Replace '{}' for {}? y(yes)/n(no)/c(custom): "
    question = question.format(bibs[0]["journal"], bibs[0]["_text"])
    action = input(question)
    try:
        return actions.get(action)()
    except TypeError:
        return manual_update_in(bibs, db_abbrev)


def update_in(bibs, db_abbrev, custom=False):
    if custom:
        abbrev = input("Insert abreviation:\n")
        db_abbrev.update(
            bibs[0]["journal"],
            abbrev
        )
    else:
        abbrev = bibs[0]["_text"]

    for i, bib in enumerate(bibs):
        bibs[i]["journal"] = abbrev

    return bibs


def update_bibs_in(grouped_bibs, db_abbrev):
    actions = {
        "y": lambda items: [update_in(bibs, db_abbrev) for bibs in items],
        "m": lambda items: [manual_update_in(bibs, db_abbrev) for bibs in items],
        "n": lambda items: items
    }
    print("\n ")
    action = input("Abbreviate everthing?" +
                   "y(yes, automatic)/m(manual)/n(do nothing)")
    grouped_bibs.sort(key=operator.itemgetter('journal'))
    grouped_by_journal = []
    for key, items in groupby(grouped_bibs, lambda i: i["journal"]):
        grouped_by_journal.append(list(items))

    if action in ("a", "m", "n"):
        updated_bibs = actions.get(action)(grouped_by_journal)
    else:
        return update_bibs_in(grouped_bibs, db_abbrev)

    updated_bibs = reduce(lambda a, b: a+b, updated_bibs)
    return updated_bibs



from __future__ import print_function
from builtins import input
from arxivcheck.arxiv import check_arxiv_published
import bibtexparser
from itertools import groupby
import operator
from functools import reduce


def update_arxivs(bibs, automatic=True):
    arxiv_id = bibs[0]["journal"].partition(":")[2]
    arxiv_title = bibs[0]["title"]

    found, published, bib_string = check_arxiv_published(arxiv_id,
                                                         abbrev_journal=False)
    bib = bibtexparser.loads(bib_string)

    if found:
        for i in range(len(bibs)):
            if automatic:
                bibs[i] = bib.entries[0]
            else:
                question = "{} >> {} ".format(arxiv_title,
                                              bib.entries[0]["title"])

                question += " was "
                if not published:
                    question += " not published. Use bib from arxiv?y/n"
                else:
                    question += " published in "
                    question += "{}.".format(bib.entries[0]["journal"])
                    question += " Replace with new bib ?y/n"
                replace = input(question)
                if replace == "y":
                    bibs[i] = bib.entries[0]

    elif not found:
        print("'{}' not founded with a id {}".format(
            arxiv_title,
            arxiv_id
        ))

    return bibs


def update_bibs_arxiv(grouped_bibs):
    actions = {
        "a": lambda items: [update_arxivs(bibs, True) for bibs in items],
        "m": lambda items: [update_arxivs(bibs, False) for bibs in items],
        "n": lambda items: items
    }

    action = input("Check if arxiv items are published?a(automatic)/m(manual)/n(do nothing)")
    grouped_bibs.sort(key=operator.itemgetter('journal'))
    grouped_by_journal = []
    for key, items in groupby(grouped_bibs, lambda i: i["journal"]):
        grouped_by_journal.append(list(items))

    if action in ("a", "m", "n"):
        updated_bibs = actions.get(action)(grouped_by_journal)

    else:
        return update_bibs_arxiv(grouped_bibs)

    updated_bibs = reduce(lambda a, b: a+b, updated_bibs)
    return updated_bibs



from setuptools import setup, find_packages

readme = open('README.md','r')
README_TEXT = readme.read()
readme.close()

setup(
    name="bibcure",
    version="0.1",
    packages = find_packages(exclude=["build",]),
    scripts=["bibcure/bin/bibcure"],
    long_description = README_TEXT,
    install_requires=["bibtexparser", "future",
                      "doitobib", "titletobib", "arxivcheck"],
    include_package_data=True,
    package_data={
        "data":["data/db_abbrev.json", "data/teste"]
    },
    license="GPLv3",
    description="Abbreviates journal names inside in a given bibtex file",
    author="Bruno Messias",
    author_email="messias.physics@gmail.com",
    download_url="https://github.com/bibcure/bibcure/archive/0.1.tar.gz",
    keywords=["bibtex", "arxiv","doi", "abbreviate", "science","scientific-journals"],

    classifiers=[
        "Development Status :: 4 - Beta",
        "License :: OSI Approved :: GNU General Public License v3 (GPLv3)",
        "Intended Audience :: Science/Research",
        "Programming Language :: Python",
        "Topic :: Text Processing :: Markup :: LaTeX",
    ],
    url="https://github.com/bibcure/bibcure"
)



from __future__ import print_function
from builtins import input
from itertools import groupby
import operator
from functools import reduce


def update_out(bibs, db_abbrev):
    is_abreviation = input(
        "'{}' is a abreviation?y(yes)n(no): ".format(bibs[0]["journal"])
    )
    if is_abreviation == "y":
        full_name = input("Insert journal name:\n")
        abreviation = bibs[0]["journal"]
    elif is_abreviation == "n":
        abreviation = input("Insert abreviation:\n")
        full_name = bibs[0]["journal"]
    else:
        return update_out(bibs, db_abbrev)
    db_abbrev.insert(full_name, abreviation)
    for i, bib in enumerate(bibs):
        bibs[i]["journal"] = abreviation
    return bibs


def update_bibs_out(grouped_bibs, db_abbrev):
    print("\nThe next items does not exist in database.")
    action = input("Manually update your database?y(yes)/n(do nothing)")
    if action == "n":
        return grouped_bibs
    elif action != "y":
        return update_bibs_out(grouped_bibs, db_abbrev)

    grouped_bibs.sort(key=operator.itemgetter('journal'))
    updated_bibs = []
    for key, items in groupby(grouped_bibs, lambda i: i["journal"]):
        updated_bibs.append(update_out(list(items), db_abbrev))

    updated_bibs = reduce(lambda a, b: a+b, updated_bibs)
    return updated_bibs
